/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/io.h"
#include "gctl/geometry.h"
#include "../lib/potential.h"

using namespace gctl;

int main(int argc, char const *argv[])
{
    mag_dipole md;
    md.M = 7.61365e+22; // 7.94e+22;
    md.n.set(point3dc(0.0, 0.0, 1.0));

    point3ds op;
    point3dc B;
    for (int i = 0; i < 181; i++)
    {
        for (int j = 0; j < 361; j++)
        {
            op.lon = -180 + 1.0*j;
            op.lat = 90 - 1.0*i;
            op.rad = GCTL_Earth_Radius;

            B = magkernel_single(md, op);

            std::cout << op.lon << " " << op.lat << " " 
                << B.x << " " << B.y << " " << B.z << " " 
                << B.module() << "\n";
        }
    }
}