/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/io.h"
#include "gctl/geometry.h"
#include "../lib/potential.h"

using namespace gctl;

int main(int argc, char const *argv[])
{
	array<vertex3dc> node;
	array<mag_triangle> mag_ele;

    gmshio fio;
    fio.init_file("data/prism_f/prism_f", gctl::Input);
    fio.set_packed(gctl::NotPacked, gctl::Input);
	fio.read_node(node);
	fio.read_element(mag_ele, node);

    // ensure the vertex ordering are all facing outside
    point3dc ref(0, 0, -5);
    vertex3dc *t_ptr;
    for (size_t i = 0; i < mag_ele.size(); i++)
    {
        if (dot(mag_ele[i].center() - ref, 
			cross(*mag_ele[i].vert[1] - *mag_ele[i].vert[0], 
			*mag_ele[i].vert[2] - *mag_ele[i].vert[0])) < 0)
        {
            t_ptr = mag_ele[i].vert[1];
            mag_ele[i].vert[1] = mag_ele[i].vert[2];
            mag_ele[i].vert[2] = t_ptr;
        }
    }
    
    array<magtri_para> mag_para;
    array<double> sus(mag_ele.size(), 0.1);

    callink_magnetic_para_earth(mag_ele, mag_para, 60.0, 30.0);

	// 设置观测点位
	array<point3dc> obsp;
	gridspace(point3dc(-30, 0, 5), point3dc(30, 0, 5), point3dc(0, -30, 5), point3dc(0, 30, 5), 121, 121, obsp);

	// 正演计算
	array<double> obsval(obsp.size());
	array<point3dc> obsgrad;
	array<tensor> obstensor;

	// 正演磁分量数据
	magobser(obsgrad, mag_ele, obsp, sus, ShortMsg);
	// 保存网格
	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].x;
	}
	gctl::save_netcdf_grid("out_magobser_tri", obsval, 121, 121, -30.0, 0.5, -30.0, 0.5, "x", "y", "Bx");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].y;
	}
	gctl::append_netcdf_grid("out_magobser_tri", obsval, "x", "y", "By");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].z;
	}
	gctl::append_netcdf_grid("out_magobser_tri", obsval, "x", "y", "Bz");

	gctl::_1d_array deltaT;
	magnetic_components2deltaT(obsgrad, deltaT, 60, 30);
	gctl::append_netcdf_grid("out_magobser_tri", deltaT, "x", "y", "DeltaT");
	return 0;
}