/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GM_TET_MESH_H
#define _GM_TET_MESH_H

#include "gctl/mesh/tet_mesh.h"

#include "gkernel_tetrahedron.h"
#include "mkernel_tetrahedron_Ren2017.h"

namespace gctl
{
	class gm_tetrahedron_mesh : public tetrahedron_mesh
	{
	protected:
		array<grav_tetrahedron> grav_ele_;
		array<gravtet_para> grav_para_;
		array<magtet_ren17> mag_ele_;
		array<magtet_para_ren17> mag_para_;

	public:
		gm_tetrahedron_mesh();
		gm_tetrahedron_mesh(std::string in_name, std::string in_info, const array<vertex3dc> &in_nodes, 
			const array<tetrahedron> &in_tets);
		virtual ~gm_tetrahedron_mesh();

		void gkernel(matrix<double> &out_kernel, const array<point3dc> &obsp, gravitational_field_type_e comp_id = Vz, 
			verbose_type_e verbose = FullMsg);
		void gkernel(matrix<double> &out_kernel, const array<point3ds> &obsp, gravitational_field_type_e comp_id = Vz, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<double> &out_obs, std::string data_name, const array<point3dc> &obsp, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<point3dc> &out_obs, std::string data_name, const array<point3dc> &obsp, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<tensor> &out_obs, std::string data_name, const array<point3dc> &obsp, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<double> &out_obs, std::string data_name, const array<point3ds> &obsp, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<point3dc> &out_obs, std::string data_name, const array<point3ds> &obsp, 
			verbose_type_e verbose = FullMsg);
		void gobser(array<tensor> &out_obs, std::string data_name, const array<point3ds> &obsp, 
			verbose_type_e verbose = FullMsg);

        void magkernel(matrix<double> &out_kernel, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);
		void magkernel(matrix<point3dc> &out_kernel, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);
		void magkernel(matrix<tensor> &out_kernel, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);

        void magobser(array<double> &out_obs, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);
        void magobser(array<point3dc> &out_obs, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);
		void magobser(array<tensor> &out_obs, std::string sus_name, const array<point3dc> &obsp, 
        	double inclina_deg, double declina_deg, verbose_type_e verbose = FullMsg);

		void magobser(array<double> &out_obs, std::string magz_name, const array<point3dc> &obsp, 
        	verbose_type_e verbose = FullMsg);
        void magobser(array<point3dc> &out_obs, std::string magz_name, const array<point3dc> &obsp, 
        	verbose_type_e verbose = FullMsg);
		void magobser(array<tensor> &out_obs, std::string magz_name, const array<point3dc> &obsp, 
        	verbose_type_e verbose = FullMsg);
	};
}

#endif // _GM_TET_MESH_H